#include "dxstdafx.h"
#include "resource.h"
#include "texturemanager.h"
#include "meshmanager.h"
#include "particlemanager.h"
#include "particlesystem.h"

CMMParticleSystem* g_ps = NULL;

void CALLBACK InitParticle( CMMParticleSystem* PSYSTEM, int PARTICLEINDEX, bool FIRSTTIME )
{
	D3DVECTOR pos;
	pos.x = -1.0f;
	pos.y = 0.0f;
	pos.z = 0.0f;
	D3DCOLOR col = 0xffffffff;
	PSYSTEM->GetParticle( PARTICLEINDEX )->SetPosition( pos );
	PSYSTEM->GetParticle( PARTICLEINDEX )->SetColor( col );
};

bool CALLBACK IsParticleAlive( CMMParticleSystem* PSYSTEM, int PARTICLEINDEX )
{
	return true;
};

void CALLBACK UpdateParticle( CMMParticleSystem* PSYSTEM, int PARTICLEINDEX )
{
	return;
};

void CALLBACK OnFrameRender( IDirect3DDevice9* pd3dDevice, double fTime, float fElapsedTime );
HRESULT CALLBACK OnCreateDevice( IDirect3DDevice9* pd3dDevice, const D3DSURFACE_DESC* pBackBufferSurfaceDesc );
void InitApp();
void DoneApp();

INT WINAPI WinMain( HINSTANCE, HINSTANCE, LPSTR, int )
{
    DXUTSetCallbackDeviceCreated( OnCreateDevice );
    DXUTSetCallbackFrameRender( OnFrameRender );

    DXUTSetCursorSettings( true, true );

    InitApp();

    DXUTInit( true, true, true );
    DXUTCreateWindow( L"FirstGame" );
    DXUTCreateDevice( D3DADAPTER_DEFAULT, true, 640, 480, NULL, NULL );

	DXUTMainLoop();

	DoneApp();

    return DXUTGetExitCode();
}

void InitApp()
{
	Textures::InitTextureManager();
	Meshes::InitMeshManager();
	Particles::InitParticleManager();
	g_ps = new CMMParticleSystem;
}

void DoneApp()
{
	SAFE_DELETE( g_ps );
	Meshes::GetMeshManager()->DeleteAll();
	Meshes::DoneMeshManager();
	Particles::GetParticleManager()->DeleteAll();
	Particles::DoneParticleManager();
	Textures::GetTextureManager()->DeleteAll();
	Textures::DoneTextureManager();
}

void CALLBACK OnFrameRender( IDirect3DDevice9* pd3dDevice, double fTime, float fElapsedTime )
{
	pd3dDevice->Clear(0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, D3DCOLOR_ARGB(0, 45, 50, 170), 1.0f, 0);

    if( SUCCEEDED( pd3dDevice->BeginScene() ) )
    {
		g_ps->Draw( pd3dDevice );
		pd3dDevice->EndScene();
		pd3dDevice->Present( NULL, NULL, NULL, NULL );
	}
}

HRESULT CALLBACK OnCreateDevice( IDirect3DDevice9* pd3dDevice, const D3DSURFACE_DESC* pBackBufferSurfaceDesc )
{
	D3DXMATRIXA16 matWorld;
	D3DXMatrixIdentity( &matWorld );
	pd3dDevice->SetTransform( D3DTS_WORLD, &matWorld );

	D3DXVECTOR3 vEyePt( 0.0f, 0.0f,-3.0f );
    D3DXVECTOR3 vLookatPt( 0.0f, 0.0f, 0.0f );
    D3DXVECTOR3 vUpVec( 0.0f, 1.0f, 0.0f );
    D3DXMATRIXA16 matView;
    D3DXMatrixLookAtLH( &matView, &vEyePt, &vLookatPt, &vUpVec );
    pd3dDevice->SetTransform( D3DTS_VIEW, &matView );

	D3DXMATRIXA16 matProj;
	D3DXMatrixPerspectiveFovLH( &matProj, D3DX_PI/4, 1.0f, 1.0f, 100.0f );
	pd3dDevice->SetTransform( D3DTS_PROJECTION, &matProj );

	pd3dDevice->SetRenderState( D3DRS_AMBIENT, 0xffffffff );

	CMMTexture* t_texture = Textures::GetTextureManager()->CreateTextureObject( TEXTURE_SIMPLE );
	t_texture->LoadTexture( pd3dDevice, L"Particle.bmp" );
	g_ps->Reset( pd3dDevice, 3, 0.05f, InitParticle, UpdateParticle, IsParticleAlive, t_texture, false );

    return S_OK;
}
